﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EAA
{
    class EnclosedArea
    {
        public List<MathFunc> Functions { get; set; }
        public List<RangeF> Ranges { get; set; }
        private List<PointF> BorderPoints { get; set; }
        public List<PointF> Intersections { get; set; }
        public List<PointF> IntraPoints { get; set; }
        public double Area { get; set; }

        public EnclosedArea(List<MathFunc> functions, List<PointF> intersections)
        {
            this.Functions = functions;
            this.Intersections = intersections;
            this.Ranges = new List<RangeF>();
            this.BorderPoints = new List<PointF>();
            this.IntraPoints = new List<PointF>();
            this.CalculateData();
            this.Area = this.GetArea(Properties.Settings.Default.dots);
        }

        private void CalculateData()
        {
            MathFunc func1 = this.Functions[0];
            MathFunc func2 = this.Functions[1];
            PointF intersec1 = this.Intersections[0];
            PointF intersec2 = this.Intersections[1];
            RangeF range1 = new RangeF(intersec1.X - func1.MidPos.X, intersec2.X - func1.MidPos.X);
            RangeF range2 = new RangeF(intersec1.X - func2.MidPos.X, intersec2.X - func2.MidPos.X);

            float midX = (intersec1.X + intersec2.X) / 2;
            float y1 = func1.GetY(midX - func1.MidPos.X) + func1.MidPos.Y;
            float y2 = func2.GetY(midX - func2.MidPos.X) + func2.MidPos.Y;
            float upperLimit;
            float lowerLimit;
            if (y1 < y2)
            {
                lowerLimit = GetExtrema(func1, range1, EAA.Form1.Extrema.Min);
                upperLimit = GetExtrema(func2, range2, EAA.Form1.Extrema.Max);
            }
            else
            {
                upperLimit = GetExtrema(func1, range1, EAA.Form1.Extrema.Max);
                lowerLimit = GetExtrema(func2, range2, EAA.Form1.Extrema.Min);
            }
            this.Ranges.Add(range1);
            this.Ranges.Add(range2);
            this.BorderPoints.Add(new PointF(intersec1.X, lowerLimit));
            this.BorderPoints.Add(new PointF(intersec2.X, upperLimit));
        }

        public void DrawInformation(Graphics g, int nr)
        {
            if (Properties.Settings.Default.colorArea)
            {
                MathFunc func1 = this.Functions[0];
                MathFunc func2 = this.Functions[1];
                for (float x = this.Intersections[0].X; x < this.Intersections[1].X; x++)
                {
                    float y1 = func1.GetY(x - func1.MidPos.X) + func1.MidPos.Y;
                    float y2 = func2.GetY(x - func2.MidPos.X) + func2.MidPos.Y;
                    PointF p1 = new PointF(x, y1);
                    PointF p2 = new PointF(x, y2);
                    g.DrawLine(Pens.LightBlue, p1, p2);
                }
            }
            float midX = (this.Intersections[0].X + this.Intersections[1].X) / 2;
            float midY = (this.BorderPoints[0].Y + this.BorderPoints[1].Y) / 2;
            g.DrawString("A" + nr, new Font("", 8, FontStyle.Italic), Brushes.Coral, new PointF(midX - 8, midY - 8));              
        }

        public float GetExtrema(MathFunc func, RangeF range, EAA.Form1.Extrema ex)
        {
            List<float> data = new List<float>();
            for (float x = range.Min; x < range.Max; x += 0.1f)
            {
                data.Add(func.GetY(x) + func.MidPos.Y);
            }
            switch (ex)
            {
                case EAA.Form1.Extrema.Max:
                    return data.Max();
                case EAA.Form1.Extrema.Min:
                    return data.Min();
            }
            return 0f;
        }

        public bool IsPointInArea(PointF point)
        {
            MathFunc func1 = this.Functions[0];
            MathFunc func2 = this.Functions[1];
            float x = point.X;
            float y = point.Y;
            float y1 = func1.GetY(x - func1.MidPos.X) + func1.MidPos.Y;
            float y2 = func2.GetY(x - func2.MidPos.X) + func2.MidPos.Y;
            if (x > this.Intersections[0].X && x < this.Intersections[1].X)
            {
                if (y1 < y2)
                {
                    if (y > y1 && y < y2)
                        return true;
                }
                else if (y > y2 && y < y1)
                    return true;
            }
            return false;
        }

        public double GetArea(int dots)
        {
            switch(Properties.Settings.Default.calcMethod)
            {
                case 1:
                    return GetAreaMonteCarlo(dots);
                case 2:
                    return GetAreaApproximation(dots);
            }
            return 0.0;
        }
        public double GetAreaApproximation(int pts)
        {
            MathFunc func1 = this.Functions[0];
            MathFunc func2 = this.Functions[1];

            PointF leftUpperPoint = BorderPoints[0];
            PointF rightLowerPoint = BorderPoints[1];
            double dx = (rightLowerPoint.X - leftUpperPoint.X);
            double sliceWidth = dx / pts;
            double area = 0;

            double x1d = leftUpperPoint.X - func1.MidPos.X;
            double x2d = leftUpperPoint.X - func2.MidPos.X;

            for (double q = 0; q < dx; q += sliceWidth)
            {
                double y1 = func1.GetY((float)(x1d + q));
                double y2 = func2.GetY((float)(x2d + q));
                area += (y1 - y2);
            }

            return Math.Abs((area+(func1.MidPos.Y-func2.MidPos.Y)*pts) * sliceWidth);
        }

        public double GetAreaMonteCarlo(int dots)
        {
            List<PointF> areaDots = new List<PointF>();
            Random rndm = new Random();
            PointF leftUpperPoint = BorderPoints[0];
            PointF rightLowerPoint = BorderPoints[1];
            int intraPoints = 0;
            float dx = (rightLowerPoint.X - leftUpperPoint.X);
            float dy = (rightLowerPoint.Y - leftUpperPoint.Y);
            for (int p = 0; p < dots; p++)
            {
                double rd = rndm.NextDouble();
                float rx = (float)(leftUpperPoint.X + rd * dx);
                rd = rndm.NextDouble();
                float ry = (float)(leftUpperPoint.Y + rd * dy);
                areaDots.Add(new PointF(rx, ry));
            }
            
            foreach (PointF dot in areaDots)
            {
                if (this.IsPointInArea(dot))
                {
                    intraPoints++;
                    this.IntraPoints.Add(dot);
                }
            }
            double ratio = (double)intraPoints / (double)dots;
            double rectangleArea = (rightLowerPoint.X - leftUpperPoint.X) * (rightLowerPoint.Y - leftUpperPoint.Y);
            double area = ratio * rectangleArea;
            return area;
        }
    }
}
